{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit mainChemForm;

{$mode objfpc}{$H+}

interface

uses
  Classes, RTTIGrids, Forms, Controls, Graphics, Grids;

type

  { TChemForm }

  TChemForm = class(TForm)
    chemGrid: TTIGrid;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    Felements: TCollection;
    procedure SetupChemGrid;
  end;

var
  ChemForm: TChemForm;

implementation

{$R *.lfm}

type
  TElementItem = class(TCollectionItem)
  private
    FNumber: cardinal;
    FName: string;
    FSymbol: string;
    FGroup: string;
  published
    property AtomicNo: cardinal read FNumber write FNumber;
    property Name: string read FName write FName;
    property Symbol: string read FSymbol write FSymbol;
    property Group: string read FGroup write FGroup;
  end;

{ TChemForm }

procedure TChemForm.FormCreate(Sender: TObject);
begin
 SetupChemGrid;
end;

procedure TChemForm.FormDestroy(Sender: TObject);
begin
  Felements.Free;
end;

procedure TChemForm.SetupChemGrid;

  procedure AddElement(aNumber: cardinal; const aName, aSymbol, aGroup: string);
  var ei: TElementItem;
  begin
    ei := TElementItem(Felements.Add);
    ei.AtomicNo:= aNumber;
    ei.Name:= aName;
    ei.Symbol:= aSymbol;
    ei.Group:= aGroup;
  end;

  procedure AddElements;
  begin
     AddElement(1, 'Hydrogen', 'H', '"I"');
     AddElement(2, 'Helium', 'He', '"II"');
     AddElement(3, 'Lithium', 'Li', 'I');
     AddElement(4, 'Beryllium', 'Be', 'II');
     AddElement(5, 'Boron', 'B', 'III');
     AddElement(6, 'Carbon', 'C', 'IV');
     AddElement(7, 'Nitrogen', 'N', 'V');
     AddElement(8, 'Oxygen', 'O', 'VI');
     AddElement(9, 'Fluorine', 'F', 'VII');
     AddElement(10,'Neon', 'Ne', 'VIII');
  end;

  procedure InitialiseChemGrid;
  begin
     Self.Height:= 310;
     Self.Width:= 380;
     chemGrid.Align:= alClient;
     chemGrid.TIOptions:= chemGrid.TIOptions + [tgoStartIndexAtOne];
     chemGrid.Options:=chemGrid.Options + [goDrawFocusSelected];
     chemGrid.FixedColor:=clMoneyGreen;
     chemGrid.DefaultDrawing:=True;
     chemGrid.AutoFillColumns:=True;
  end;

begin
 Felements := TCollection.Create(TElementItem);
 AddElements;
 InitialiseChemGrid;
 chemGrid.ListObject := Felements;
end;

end.

