{ Copyright (C) 2012 H Page-Clark

  This source is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.

  This code is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
  details.

  A copy of the GNU General Public License is available on the World Wide Web
  at <http://www.gnu.org/copyleft/gpl.html>. You can also obtain it by writing
  to the Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
  MA 02111-1307, USA. }

unit stats_main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Dialogs, StdCtrls,
  EditBtn, ExtCtrls, strutils;

type

  { TForm1 }

  TForm1 = class(TForm)
    FileNameEdit1: TFileNameEdit;
    lblDirections: TLabel;
    lbStats: TListBox;
    memPreview: TMemo;
    Panel1: TPanel;
    Splitter1: TSplitter;
    procedure FileNameEdit1AcceptFileName(Sender: TObject; var Value: String);
  private
    FFileName: string;
    function CharCount(const aText: string): integer;
    procedure ComputeStats;
    procedure DisplayFileInfo;
    function WordCount(const aText: string): integer;
    function ParseToWords(const aString: string; separatorsSet: TSysCharSet): TStringList;
    function IsTextFile(aFileName: string): boolean;
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.FileNameEdit1AcceptFileName(Sender: TObject; var Value: String
  );
begin
 FFileName:= UTF8ToSys(Value);
 if not IsTextFile(FFileName)
  then case QuestionDlg('Caution!',
  Value+' does not look like a text file'+sLineBreak+'What do you want to do?',
  mtWarning,
  [mrYes,'Open file anyway', mrNo,'Look for a different file', mrCancel],
  0) of
      mrNo: Exit;
      mrYes: DisplayFileInfo;
     end // case QuestionDlg
 else DisplayFileInfo;
end;

function TForm1.CharCount(const aText: string): integer;
var c: Char;
begin
  Result := 0;
  for c in aText do
   if (Ord(c) > 32) then inc(Result);
end;

procedure TForm1.ComputeStats;
var c, w, m: integer;
    start: TDateTime;
begin
  if (FFileName = EmptyStr) then Exit;
  start := Now;
  lbStats.Items.Clear;
  lbStats.Items.Add(' ' + SysToUTF8(FFileName));
  lbStats.Items.Add('');
  c := CharCount(memPreview.Text);
  w := WordCount(memPreview.Text);
  if (w = 0) then m := 0
   else m := c div w;
  lbStats.Items.Add(' Line count: '+FormatFloat(',#', memPreview.Lines.Count));
  lbStats.Items.Add(Format(' Mean word length is: %d', [m]));
  lbStats.Items.Add(' Word count: '+FormatFloat(',#', w));
  lbStats.Items.Add(' Character count: '+FormatFloat(',#', c));
  lbStats.Items.Add(' File size is: '+FormatFloat(',#', FileSize(FFileName))+' bytes');
  lbStats.Items.Add(Format(' Calculations took %s secs',[FormatDateTime('s.zzz', Now - start)]));
end;

procedure TForm1.DisplayFileInfo;
begin
  memPreview.Lines.Clear;
  memPreview.Lines.LoadFromFile(FFileName);
  ComputeStats;
end;

function TForm1.WordCount(const aText: string): integer;
var sl: TStringList;
begin
  Result := 0;
  if aText = EmptyStr then Exit;
  try
    sl := ParseToWords(aText,[]);
    Result := sl.Count;
  finally
    sl.Free;
  end;
end;

function TForm1.ParseToWords(const aString: string;
                             separatorsSet: TSysCharSet): TStringList;
var
    buildingAWord: boolean = False;
    s: string = '';
    c: Char;
begin
  if (Length(aString) = 0)
   then begin
          Result := nil;
          Exit;
        end;

  if (separatorsSet = [])
   then separatorsSet:= WordDelimiters;

  result := TStringList.Create;
  for c in aString do
   case (c in separatorsSet) of
    False: begin
            if not buildingAWord
             then buildingAWord:= True;
            s := s + c;
           end;
    True:  begin
            if buildingAWord
             then begin
                    buildingAWord:= False;
                    Result.Add(s);
                    s := EmptyStr;
                  end;
           end;
   end; // case
  if (s <> EmptyStr)
   then Result.Add(s);
end;

function TForm1.IsTextFile(aFileName: string): boolean;
const BufSize = 1024;
var inf: file;
    buffer: array[1..BufSize] of byte;
    numRead: int64;

  function IsUTF8OrXML: boolean;
  begin
    Result := False;
    if buffer[1] = $3C then Result:= True;
    if (buffer[1] = $00) and (buffer[2] = $3C) then Result := True;
    if (buffer[1] = $FF) and (buffer[2] = $FE) then Result := True;
    if (buffer[1] = $FE) and (buffer[2] = $FF) then Result := True;
    if (buffer[1] = $EF) and (buffer[2] = $BB) and (buffer[3] = $BF)
     then Result := True;
  end;

  function IsASCIIfile: boolean;
  var index: integer;
  begin
    Result := True;
    for index := 1 to numRead do
     if Result then
      case buffer[index] of
       9, 10, 12, 13: ;
       32..126: ;
       else Result := False;
      end
     else Break;
  end;

  function IsBinary: boolean;
  var zeroCount: integer = 0;
      currZero: boolean;
      index: integer;
  begin
   Result := False;
   if (buffer[1] = Ord('M')) and (buffer[2] = Ord('Z'))
    then Exit(True);
   for index := 1 to numread do
    begin
      currZero:= buffer[index] = 0;
      case currZero of
       False: zeroCount:= 0;
       True: inc(zeroCount);
      end;
      if (zeroCount >= 4) then
       begin
         Result := True;
         Break;
       end;
    end;
  end;

begin
  Filemode := 0;
  AssignFile(inf, aFileName);
  try
   Reset(inf, 1);
  except
   begin ShowMessageFmt('File %s could not be opened!',[aFileName]);
         Result := False;
         Exit;
   end;
  end;
  BlockRead(inf, buffer, SizeOf(buffer), numRead);
  CloseFile(inf);
  result := IsASCIIfile or IsUTF8OrXML and not IsBinary;
end;

end.

